@@ -1,48 +1,36 @@
 /*
- * Copyright 2013 Albert Vaca <albertvaka@gmail.com>
- * Copyright 2015 Aleix Pol i Gonzalez <aleixpol@kde.org>
+ * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
+ * SPDX-FileCopyrightText: 2015 Aleix Pol i Gonzalez <aleixpol@kde.org>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "filetransferjob.h"
 #include "daemon.h"
 #include <core_debug.h>
 
-#include <qalgorithms.h>
-#include <QFileInfo>
 #include <QDebug>
+#include <QFileInfo>
+#include <QNetworkAccessManager>
+#include <qalgorithms.h>
 
+#include <KFileUtils>
 #include <KLocalizedString>
 
-FileTransferJob::FileTransferJob(const NetworkPacket* np, const QUrl& destination)
+FileTransferJob::FileTransferJob(const NetworkPacket *np, const QUrl &destination)
     : KJob()
     , m_origin(np->payload())
     , m_reply(nullptr)
     , m_from(QStringLiteral("KDE Connect"))
     , m_destination(destination)
-    , m_speedBytes(0)
     , m_written(0)
     , m_size(np->payloadSize())
     , m_np(np)
+    , m_autoRename(false)
 {
     Q_ASSERT(m_origin);
-    //Disabled this assert: QBluetoothSocket doesn't report "->isReadable() == true" until it's connected
-    //Q_ASSERT(m_origin->isReadable());
+    // Disabled this assert: QBluetoothSocket doesn't report "->isReadable() == true" until it's connected
+    // Q_ASSERT(m_origin->isReadable());
     if (m_destination.scheme().isEmpty()) {
         qCWarning(KDECONNECT_CORE) << "Destination QUrl" << m_destination << "lacks a scheme. Setting its scheme to 'file'.";
         m_destination.setScheme(QStringLiteral("file"));
@@ -55,16 +43,23 @@ FileTransferJob::FileTransferJob(const NetworkPacket* np, const QUrl& destinatio
 void FileTransferJob::start()
 {
     QMetaObject::invokeMethod(this, "doStart", Qt::QueuedConnection);
-    //qCDebug(KDECONNECT_CORE) << "FileTransferJob start";
+    // qCDebug(KDECONNECT_CORE) << "FileTransferJob start";
 }
 
 void FileTransferJob::doStart()
 {
     if (m_destination.isLocalFile() && QFile::exists(m_destination.toLocalFile())) {
-        setError(2);
-        setErrorText(i18n("Filename already present"));
-        emitResult();
-        return;
+        if (m_autoRename) {
+            QFileInfo fileInfo(m_destination.toLocalFile());
+            QString path = fileInfo.path();
+            QString fileName = fileInfo.fileName();
+            m_destination.setPath(path + QStringLiteral("/") + KFileUtils::suggestName(QUrl(path), fileName), QUrl::DecodedMode);
+        } else {
+            setError(2);
+            setErrorText(i18n("Filename already present"));
+            emitResult();
+            return;
+        }
     }
 
     if (m_origin->bytesAvailable())
@@ -99,12 +94,7 @@ void FileTransferJob::startTransfer()
 
         m_written = bytesSent;
     });
-#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
-    connect(m_reply, static_cast<void (QNetworkReply::*)(QNetworkReply::NetworkError)>(&QNetworkReply::error),
-            this, &FileTransferJob::transferFailed);
-#else
     connect(m_reply, &QNetworkReply::errorOccurred, this, &FileTransferJob::transferFailed);
-#endif
     connect(m_reply, &QNetworkReply::finished, this, &FileTransferJob::transferFinished);
 }
 
@@ -120,12 +110,12 @@ void FileTransferJob::transferFailed(QNetworkReply::NetworkError error)
 
 void FileTransferJob::transferFinished()
 {
-    //TODO: MD5-check the file
+    // TODO: MD5-check the file
     if (m_size == m_written) {
         qCDebug(KDECONNECT_CORE) << "Finished transfer" << m_destination;
         emitResult();
     } else {
-        qCDebug(KDECONNECT_CORE) << "Received incomplete file ("<< m_written << "/" << m_size << "bytes ), deleting";
+        qCDebug(KDECONNECT_CORE) << "Received incomplete file (" << m_written << "/" << m_size << "bytes ), deleting";
 
         deleteDestinationFile();
 
@@ -155,3 +145,5 @@ bool FileTransferJob::doKill()
 
     return true;
 }
+
+#include "moc_filetransferjob.cpp"
