@@ -19,21 +19,19 @@
 #include <QStandardPaths>
 #include <QThread>
 #include <QUuid>
-#include <QtCrypto>
 
 #include "core_debug.h"
 #include "daemon.h"
 #include "dbushelper.h"
+#include "deviceinfo.h"
+#include "pluginloader.h"
+#include "sslhelper.h"
 
 const QFile::Permissions strictPermissions = QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser;
 
 struct KdeConnectConfigPrivate {
-    // The Initializer object sets things up, and also does cleanup when it goes out of scope
-    // Note it's not being used anywhere. That's intended
-    QCA::Initializer m_qcaInitializer;
-
-    QCA::PrivateKey m_privateKey;
-    QSslCertificate m_certificate; // Use QSslCertificate instead of QCA::Certificate due to compatibility with QSslSocket
+    QSslKey m_privateKey;
+    QSslCertificate m_certificate;
 
     QSettings *m_config;
     QSettings *m_trustedDevices;
@@ -45,19 +43,6 @@ struct KdeConnectConfigPrivate {
 
 static QString getDefaultDeviceName()
 {
-#ifdef SAILFISHOS
-    const QString hwReleaseFile = QStringLiteral("/etc/hw-release");
-    // QSettings will crash if the file does not exist or can be created, like in this case by us in /etc.
-    // E.g. in the SFOS SDK Emulator there is no such file, so check before to protect against the crash.
-    if (QFile::exists(hwReleaseFile)) {
-        QSettings hwRelease(hwReleaseFile, QSettings::IniFormat);
-        auto hwName = hwRelease.value(QStringLiteral("NAME")).toString();
-        if (!hwName.isEmpty()) {
-            return hwName;
-        }
-    }
-#endif
-
     return QHostInfo::localHostName();
 }
 
@@ -70,15 +55,6 @@ KdeConnectConfig &KdeConnectConfig::instance()
 KdeConnectConfig::KdeConnectConfig()
     : d(new KdeConnectConfigPrivate)
 {
-    // qCDebug(KDECONNECT_CORE) << "QCA supported capabilities:" << QCA::supportedFeatures().join(",");
-    if (!QCA::isSupported("rsa")) {
-        qCritical() << "Could not find support for RSA in your QCA installation";
-        Daemon::instance()->reportError(i18n("KDE Connect failed to start"),
-                                        i18n("Could not find support for RSA in your QCA installation. If your "
-                                             "distribution provides separate packets for QCA-ossl and QCA-gnupg, "
-                                             "make sure you have them installed and try again."));
-    }
-
     // Make sure base directory exists
     QDir().mkpath(baseConfigDir().path());
 
@@ -86,8 +62,7 @@ KdeConnectConfig::KdeConnectConfig()
     d->m_config = new QSettings(baseConfigDir().absoluteFilePath(QStringLiteral("config")), QSettings::IniFormat);
     d->m_trustedDevices = new QSettings(baseConfigDir().absoluteFilePath(QStringLiteral("trusted_devices")), QSettings::IniFormat);
 
-    loadPrivateKey();
-    loadCertificate();
+    loadOrGeneratePrivateKeyAndCertificate(privateKeyPath(), certificatePath());
 
     if (name().isEmpty()) {
         setName(getDefaultDeviceName());
@@ -105,25 +80,21 @@ void KdeConnectConfig::setName(const QString &name)
     d->m_config->sync();
 }
 
-QString KdeConnectConfig::deviceType()
+DeviceType KdeConnectConfig::deviceType()
 {
-#ifdef SAILFISHOS
-    return QStringLiteral("phone");
-#else
     const QByteArrayList platforms = qgetenv("PLASMA_PLATFORM").split(':');
 
     if (platforms.contains("phone")) {
-        return QStringLiteral("phone");
+        return DeviceType::Phone;
     } else if (platforms.contains("tablet")) {
-        return QStringLiteral("tablet");
+        return DeviceType::Tablet;
     } else if (platforms.contains("mediacenter")) {
-        return QStringLiteral("tv");
+        return DeviceType::Tv;
     }
 
     // TODO non-Plasma mobile platforms
 
-    return QStringLiteral("desktop");
-#endif
+    return DeviceType::Desktop;
 }
 
 QString KdeConnectConfig::deviceId()
@@ -146,6 +117,19 @@ QSslCertificate KdeConnectConfig::certificate()
     return d->m_certificate;
 }
 
+DeviceInfo KdeConnectConfig::deviceInfo()
+{
+    const auto incoming = PluginLoader::instance()->incomingCapabilities();
+    const auto outgoing = PluginLoader::instance()->outgoingCapabilities();
+    return DeviceInfo(deviceId(),
+                      certificate(),
+                      name(),
+                      deviceType(),
+                      NetworkPacket::s_protocolVersion,
+                      QSet(incoming.begin(), incoming.end()),
+                      QSet(outgoing.begin(), outgoing.end()));
+}
+
 QDir KdeConnectConfig::baseConfigDir()
 {
     QString configPath = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation);
@@ -159,27 +143,54 @@ QStringList KdeConnectConfig::trustedDevices()
     return list;
 }
 
-void KdeConnectConfig::addTrustedDevice(const QString &id, const QString &name, const QString &type)
+void KdeConnectConfig::addTrustedDevice(const DeviceInfo &deviceInfo)
 {
-    d->m_trustedDevices->beginGroup(id);
-    d->m_trustedDevices->setValue(QStringLiteral("name"), name);
-    d->m_trustedDevices->setValue(QStringLiteral("type"), type);
+    d->m_trustedDevices->beginGroup(deviceInfo.id);
+    d->m_trustedDevices->setValue(QStringLiteral("name"), deviceInfo.name);
+    d->m_trustedDevices->setValue(QStringLiteral("type"), deviceInfo.type.toString());
+    QString certString = QString::fromLatin1(deviceInfo.certificate.toPem());
+    d->m_trustedDevices->setValue(QStringLiteral("certificate"), certString);
     d->m_trustedDevices->endGroup();
     d->m_trustedDevices->sync();
 
-    QDir().mkpath(deviceConfigDir(id).path());
+    QDir().mkpath(deviceConfigDir(deviceInfo.id).path());
 }
 
-KdeConnectConfig::DeviceInfo KdeConnectConfig::getTrustedDevice(const QString &id)
+void KdeConnectConfig::updateTrustedDeviceInfo(const DeviceInfo &deviceInfo)
+{
+    if (!trustedDevices().contains(deviceInfo.id)) {
+        // do not store values for untrusted devices (it would make them trusted)
+        return;
+    }
+
+    d->m_trustedDevices->beginGroup(deviceInfo.id);
+    d->m_trustedDevices->setValue(QStringLiteral("name"), deviceInfo.name);
+    d->m_trustedDevices->setValue(QStringLiteral("type"), deviceInfo.type.toString());
+    d->m_trustedDevices->endGroup();
+    d->m_trustedDevices->sync();
+}
+
+QSslCertificate KdeConnectConfig::getTrustedDeviceCertificate(const QString &id)
+{
+    d->m_trustedDevices->beginGroup(id);
+    QString certString = d->m_trustedDevices->value(QStringLiteral("certificate"), QString()).toString();
+    d->m_trustedDevices->endGroup();
+    return QSslCertificate(certString.toLatin1());
+}
+
+DeviceInfo KdeConnectConfig::getTrustedDevice(const QString &id)
 {
     d->m_trustedDevices->beginGroup(id);
 
-    KdeConnectConfig::DeviceInfo info;
-    info.deviceName = d->m_trustedDevices->value(QStringLiteral("name"), QLatin1String("unnamed")).toString();
-    info.deviceType = d->m_trustedDevices->value(QStringLiteral("type"), QLatin1String("unknown")).toString();
+    QString certString = d->m_trustedDevices->value(QStringLiteral("certificate"), QString()).toString();
+    QSslCertificate certificate(certString.toLatin1());
+    QString name = d->m_trustedDevices->value(QStringLiteral("name"), QLatin1String("unnamed")).toString();
+    QString typeString = d->m_trustedDevices->value(QStringLiteral("type"), QLatin1String("unknown")).toString();
+    DeviceType type = DeviceType::FromString(typeString);
 
     d->m_trustedDevices->endGroup();
-    return info;
+
+    return DeviceInfo(id, certificate, name, type);
 }
 
 void KdeConnectConfig::removeTrustedDevice(const QString &deviceId)
@@ -235,56 +246,46 @@ QDir KdeConnectConfig::pluginConfigDir(const QString &deviceId, const QString &p
     return QDir(pluginConfigDir);
 }
 
-void KdeConnectConfig::loadPrivateKey()
+bool KdeConnectConfig::loadPrivateKey(const QString &keyPath)
 {
-    QString keyPath = privateKeyPath();
     QFile privKey(keyPath);
-
-    bool needsToGenerateKey = false;
     if (privKey.exists() && privKey.open(QIODevice::ReadOnly)) {
-        QCA::ConvertResult result;
-        d->m_privateKey = QCA::PrivateKey::fromPEM(QString::fromLatin1(privKey.readAll()), QCA::SecureArray(), &result);
-        if (result != QCA::ConvertResult::ConvertGood) {
-            qCWarning(KDECONNECT_CORE) << "Private key from" << keyPath << "is not valid";
-            needsToGenerateKey = true;
+        d->m_privateKey = QSslKey(privKey.readAll(), QSsl::KeyAlgorithm::Rsa);
+        if (d->m_privateKey.isNull()) {
+            qCWarning(KDECONNECT_CORE) << "Private key from" << keyPath << "is not valid!";
         }
-    } else {
-        needsToGenerateKey = true;
     }
+    return d->m_privateKey.isNull();
+}
+
+bool KdeConnectConfig::loadCertificate(const QString &certPath)
+{
+    QFile cert(certPath);
+    if (cert.exists() && cert.open(QIODevice::ReadOnly)) {
+        d->m_certificate = QSslCertificate(cert.readAll());
+        if (d->m_certificate.isNull()) {
+            qCWarning(KDECONNECT_CORE) << "Certificate from" << certPath << "is not valid";
+        }
+    }
+    return d->m_certificate.isNull();
+}
+
+void KdeConnectConfig::loadOrGeneratePrivateKeyAndCertificate(const QString &keyPath, const QString &certPath)
+{
+    bool needsToGenerateKey = loadPrivateKey(keyPath);
+    bool needsToGenerateCert = needsToGenerateKey || loadCertificate(certPath);
 
     if (needsToGenerateKey) {
         generatePrivateKey(keyPath);
     }
+    if (needsToGenerateCert) {
+        generateCertificate(certPath);
+    }
 
     // Extra security check
     if (QFile::permissions(keyPath) != strictPermissions) {
         qCWarning(KDECONNECT_CORE) << "Warning: KDE Connect private key file has too open permissions " << keyPath;
     }
-}
-
-void KdeConnectConfig::loadCertificate()
-{
-    QString certPath = certificatePath();
-    QFile cert(certPath);
-
-    bool needsToGenerateCert = false;
-    if (cert.exists() && cert.open(QIODevice::ReadOnly)) {
-        auto loadedCerts = QSslCertificate::fromPath(certPath);
-        if (loadedCerts.empty()) {
-            qCWarning(KDECONNECT_CORE) << "Certificate from" << certPath << "is not valid";
-            needsToGenerateCert = true;
-        } else {
-            d->m_certificate = loadedCerts.at(0);
-        }
-    } else {
-        needsToGenerateCert = true;
-    }
-
-    if (needsToGenerateCert) {
-        generateCertificate(certPath);
-    }
-
-    // Extra security check
     if (QFile::permissions(certPath) != strictPermissions) {
         qCWarning(KDECONNECT_CORE) << "Warning: KDE Connect certificate file has too open permissions " << certPath;
     }
@@ -294,16 +295,19 @@ void KdeConnectConfig::generatePrivateKey(const QString &keyPath)
 {
     qCDebug(KDECONNECT_CORE) << "Generating private key";
 
-    bool error = false;
-
-    d->m_privateKey = QCA::KeyGenerator().createRSA(2048);
+    d->m_privateKey = SslHelper::generateRsaPrivateKey();
+    if (d->m_privateKey.isNull()) {
+        qCritical() << "Could not generate the private key";
+        Daemon::instance()->reportError(i18n("KDE Connect failed to start"), i18n("Could not generate the private key."));
+    }
 
     QFile privKey(keyPath);
+    bool error = false;
     if (!privKey.open(QIODevice::ReadWrite | QIODevice::Truncate)) {
         error = true;
     } else {
         privKey.setPermissions(strictPermissions);
-        int written = privKey.write(d->m_privateKey.toPEM().toLatin1());
+        int written = privKey.write(d->m_privateKey.toPem());
         if (written <= 0) {
             error = true;
         }
@@ -318,32 +322,18 @@ void KdeConnectConfig::generateCertificate(const QString &certPath)
 {
     qCDebug(KDECONNECT_CORE) << "Generating certificate";
 
-    bool error = false;
-
     QString uuid = QUuid::createUuid().toString();
     DBusHelper::filterNonExportableCharacters(uuid);
     qCDebug(KDECONNECT_CORE) << "My id:" << uuid;
 
-    // FIXME: We only use QCA here to generate the cert and key, would be nice to get rid of it completely.
-    // The same thing we are doing with QCA could be done invoking openssl (although it's potentially less portable):
-    // openssl req -new -x509 -sha256 -newkey rsa:2048 -nodes -keyout privateKey.pem -days 3650 -out certificate.pem -subj "/O=KDE/OU=KDE
-    // Connect/CN=_e6e29ad4_2b31_4b6d_8f7a_9872dbaa9095_"
-
-    QCA::CertificateOptions certificateOptions = QCA::CertificateOptions();
-    QDateTime startTime = QDateTime::currentDateTime().addYears(-1);
-    QDateTime endTime = startTime.addYears(10);
-    QCA::CertificateInfo certificateInfo;
-    certificateInfo.insert(QCA::CommonName, uuid);
-    certificateInfo.insert(QCA::Organization, QStringLiteral("KDE"));
-    certificateInfo.insert(QCA::OrganizationalUnit, QStringLiteral("Kde connect"));
-    certificateOptions.setInfo(certificateInfo);
-    certificateOptions.setFormat(QCA::PKCS10);
-    certificateOptions.setSerialNumber(QCA::BigInteger(10));
-    certificateOptions.setValidityPeriod(startTime, endTime);
-
-    d->m_certificate = QSslCertificate(QCA::Certificate(certificateOptions, d->m_privateKey).toPEM().toLatin1());
+    d->m_certificate = SslHelper::generateSelfSignedCertificate(d->m_privateKey, uuid);
+    if (d->m_certificate.isNull()) {
+        qCritical() << "Could not generate a certificate";
+        Daemon::instance()->reportError(i18n("KDE Connect failed to start"), i18n("Could not generate the device certificate."));
+    }
 
     QFile cert(certPath);
+    bool error = false;
     if (!cert.open(QIODevice::ReadWrite | QIODevice::Truncate)) {
         error = true;
     } else {
